/********************************************************************\

  Name:         rpc_srvr.c
  Created by:   Stefan Ritt

  Contents:     A simple RPC server program. Implements a function
                rpc_test, which simply doubles some parameters.
                This function can be calles remotely by rpc_clnt.


  Revision history
  ------------------------------------------------------------------
  date         by    modification
  ---------    ---   ------------------------------------------------
  02-JAN-96    SR    created


\********************************************************************/

#include "midas.h"
#include <stdio.h>

#define RPC_TEST RPC_MIN_ID

RPC_LIST rpc_list[] = {

  { RPC_TEST,
    {{TID_BYTE,       RPC_IN}, 
     {TID_WORD,       RPC_IN}, 
     {TID_INT,        RPC_IN}, 
     {TID_FLOAT,      RPC_IN}, 
     {TID_DOUBLE,     RPC_IN}, 
     {TID_BYTE,       RPC_OUT}, 
     {TID_WORD,       RPC_OUT}, 
     {TID_INT,        RPC_OUT}, 
     {TID_FLOAT,      RPC_OUT}, 
     {TID_DOUBLE,     RPC_OUT}, 
     {0} }},

  { 0 }

};

/*------------------------------------------------------------------*/

/* just a small test routine which doubles numbers */
INT rpc_test(BYTE b, WORD w, INT i, float f, double d,
             BYTE *b1, WORD *w1, INT *i1, float *f1, double *d1)
{
  printf("rpc_test: %d %d %d %1.1f %1.1lf\n", b, w, i, f, d);

  *b1 = b*2;
  *w1 = w*2;
  *i1 = i*2;
  *f1 = f*2;
  *d1 = d*2;

  return 1;
}

INT rpc_dispatch(INT index, void *prpc_param[])
{
INT status;

  switch (index)
    {
    case RPC_TEST:
      status = rpc_test(CBYTE(0), CWORD(1), CINT(2), CFLOAT(3), CDOUBLE(4),
                        CPBYTE(5), CPWORD(6), CPINT(7), CPFLOAT(8), CPDOUBLE(9));
      break;
    
    default:
      cm_msg(MERROR, "rpc_dispatch", "received unrecognized command");
    }

  return status;
}

/*------------------------------------------------------------------*/

main()
{
INT status, port;

  port = 1750;
  status = rpc_register_server(ST_SINGLE, "", &port, NULL);
  if (status != RPC_SUCCESS)
    {
    printf("Cannot start server");
    return 0;
    }

  rpc_register_functions(rpc_list, rpc_dispatch);

  while (cm_yield(1000) != RPC_SHUTDOWN);

  rpc_server_shutdown();

  return 1;
}
